/*
 * Created on 30-sep-2008
 */
package be.AM4DP;

import java.io.IOException;
import java.util.*;
import java.util.logging.*;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.*;
import be.SIRAPRISE.messages.*;
import be.SIRAPRISE.util.Localizer;
import be.SIRAPRISE.util.Signer;

/**
 * Class that can be used to define methods that can commonly be used by all AM4DP web client functions
 * 
 * @author Erwin Smout
 */
abstract class AM4DPWebClientFunction extends OnLinePresentationFunction {

	/**
	 * 
	 */
	private static SiraPriseServer host;

	/**
	 * 
	 */
	private static final String SIGNERCLASSNAME = "SIGNERCLASSNAME"; //$NON-NLS-1$

	/**
	 * The Signer that will be used when communicating with a AM4DP server
	 */
	private static Signer webClientSigner;

	/**
	 * The webclient's configured client name for obtaining database connections
	 */
	final static String clientName = "AM4DPWEBCLIENT"; //$NON-NLS-1$

	/**
	 * The set of signing algorithm names usable by the client
	 */
	static Set<String> clientSigningAlgorithmNames;

	/**
	 * The set of usable Cipher names
	 */
	static Set<String> cryptoAlgorithmNames;

	/**
	 * 
	 */
	static {
		Properties properties = AM4DPWebClientFunctionProperties.getInstance();

		clientSigningAlgorithmNames = DBConnectionProperties.getSigningAlgorithmNameSet(properties.getProperty(DBConnectionProperties.SIGNINGALGORITHMS, "")); //$NON-NLS-1$
		cryptoAlgorithmNames = DBConnectionProperties.getCryptoAlgorithmNameSet(properties.getProperty(DBConnectionProperties.CRYPTOALGORITHMS, "")); //$NON-NLS-1$
		host = new SiraPriseServer(properties.getProperty(DBConnectionProperties.HOST, DBConnectionProperties.HOSTDEFAULT), Integer.parseInt(properties.getProperty(DBConnectionProperties.PORT, DBConnectionProperties.PORTDEFAULT)), Integer.parseInt(properties.getProperty(DBConnectionProperties.MONITORPORT, DBConnectionProperties.MONITORPORTDEFAULT)));

		String siraPriseWebClientSignerClassName = properties.getProperty(SIGNERCLASSNAME, WebClientSigner.class.getName());
		try {
			webClientSigner = (Signer) AM4DPWebClientFunction.class.getClassLoader().loadClass(siraPriseWebClientSignerClassName).newInstance();
		} catch (Exception e) {
			Logger.getLogger(AM4DPWebClientFunction.class.getName()).log(Level.INFO, "Failed to load Signer.  Using default signer instead.", e); //$NON-NLS-1$
			webClientSigner = WebClientSigner.getInstance();
		}

		Logger.getLogger(AM4DPWebClientFunction.class.getName()).info("AM4DP web client connecting to host/port " + host); //$NON-NLS-1$
		Logger.getLogger(AM4DPWebClientFunction.class.getName()).info("AM4DP web client employing Signer " + webClientSigner.getClass().getName()); //$NON-NLS-1$
		Logger.getLogger(AM4DPWebClientFunction.class.getName()).info("AM4DP web client employing signatures set " + clientSigningAlgorithmNames.toString()); //$NON-NLS-1$
		Logger.getLogger(AM4DPWebClientFunction.class.getName()).info("AM4DP web client employing ciphers set " + cryptoAlgorithmNames.toString()); //$NON-NLS-1$
	}

	/**
	 * Gets the applicable presentation layer exception to report the given kind of server error to the user
	 * 
	 * @param e
	 *            The original error message read by the client
	 * @return The applicable WAAR exception object
	 */
	final static ASPLApplicationException getASPLApplicationException (ErrorMessageException e) {
		ServerErrorMessage serverErrorMessage = e.getServerErrorMessage();
		switch (serverErrorMessage.getMessageTypeID()) {
			case ConstraintViolatedErrorMessageType.MESSAGETYPEID: {
				return new DBConstraintViolationException(serverErrorMessage);
			}
			case NonSiraPriseErrorMessageType.MESSAGETYPEID: {
				return new DBServerRuntimeException(serverErrorMessage);
			}
			case SiraPriseErrorMessageType.MESSAGETYPEID: {
				return new DBSiraPriseRuntimeException(serverErrorMessage);
			}
			default: {

			}
		}
		return new DBProblem(e);
	}

	/**
	 * Gets a DBConnection for this function with the properties configured for AM4DPWebClientFunctions
	 * 
	 * @return a DBConnection for this function with the properties configured for AM4DPWebClientFunctions
	 * @throws NoConnectionException
	 *             If no connection could be established
	 */
	final DBConnection getDBConnection ( ) throws NoConnectionException {
		try {
			return new OneShotDBConnection(host, clientSigningAlgorithmNames, clientName, cryptoAlgorithmNames, webClientSigner);
		} catch (IOException e) {
			log(e);
			throw new NoConnectionException(e);
		} catch (CommunicationProtocolException e) {
			log(e);
			throw new NoConnectionException(e);
		} catch (DBException e) {
			log(e);
			throw new NoConnectionException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.OnLinePresentationFunction#getApplicationLocalizer()
	 */
	@Override
	protected final Localizer getApplicationLocalizer ( ) {
		return I18N.getInstance();
	}
}